@@ -2,7 +2,7 @@
 
  gg_extras.c -- Gaia extra functions support
     
- version 4.3, 2015 June 29
+ version 5.0, 2020 August 1
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2012-2015
+Portions created by the Initial Developer are Copyright (C) 2012-2020
 the Initial Developer. All Rights Reserved.
 
 Contributor(s):
@@ -60,12 +60,19 @@ the terms of any one of the MPL, the GPL or the LGPL.
 #include <spatialite/gaiageo.h>
 
 #if defined(_WIN32) && !defined(__MINGW32__)
+#if defined(_MSC_VER) && _MSC_VER < 1800
+/*
+/ only when using an obsolete MSVC (< 2013)
+/ we need to define an internal replacement
+/ implementig C99 rint()
+*/
 static double
 rint (double x)
 {
     return floor (x + 0.5);
 }
 #endif
+#endif
 
 static void
 auxGridSnapPoint (int dimension_model, gaiaPointPtr pt, gaiaGeomCollPtr result,
@@ -757,8 +764,7 @@ get_grid_base (double min_x, double min_y, double origin_x, double origin_y,
 
 static gaiaGeomCollPtr
 gaiaSquareGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
-		      double origin_x, double origin_y, double size,
-		      int edges_only)
+		      double origin_x, double origin_y, double size, int mode)
 {
 /* creating a regular grid [Square cells] */
     double min_x;
@@ -823,7 +829,7 @@ gaiaSquareGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 		  {
 		      /* ok, inserting a valid cell */
 		      count++;
-		      if (edges_only)
+		      if (mode > 0)
 			{
 			    /* multilinestring */
 			    ln = gaiaAddLinestringToGeomColl (result, 2);
@@ -839,6 +845,14 @@ gaiaSquareGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 			    gaiaSetPoint (ln->Coords, 0, x4, y4);
 			    gaiaSetPoint (ln->Coords, 1, x1, y1);
 			}
+		      else if (mode < 0)
+			{
+			    /* multipoint */
+			    gaiaAddPointToGeomColl (result, x1, y1);
+			    gaiaAddPointToGeomColl (result, x2, y2);
+			    gaiaAddPointToGeomColl (result, x3, y3);
+			    gaiaAddPointToGeomColl (result, x4, y4);
+			}
 		      else
 			{
 			    /* polygon */
@@ -867,7 +881,7 @@ gaiaSquareGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 	  gaiaFreeGeomColl (result);
 	  return NULL;
       }
-    if (!edges_only)
+    if (mode == 0)
       {
 	  result->DeclaredType = GAIA_MULTIPOLYGON;
 	  return result;
@@ -880,24 +894,25 @@ gaiaSquareGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 	result = gaiaUnaryUnion (item);
     gaiaFreeGeomColl (item);
     result->Srid = geom->Srid;
-    result->DeclaredType = GAIA_LINESTRING;
+    if (mode < 0)
+	result->DeclaredType = GAIA_MULTIPOINT;
+    else
+	result->DeclaredType = GAIA_MULTILINESTRING;
     return result;
 }
 
 GAIAGEO_DECLARE gaiaGeomCollPtr
 gaiaSquareGrid (gaiaGeomCollPtr geom, double origin_x, double origin_y,
-		double size, int edges_only)
+		double size, int mode)
 {
-    return gaiaSquareGridCommon (NULL, geom, origin_x, origin_y, size,
-				 edges_only);
+    return gaiaSquareGridCommon (NULL, geom, origin_x, origin_y, size, mode);
 }
 
 GAIAGEO_DECLARE gaiaGeomCollPtr
 gaiaSquareGrid_r (const void *p_cache, gaiaGeomCollPtr geom, double origin_x,
-		  double origin_y, double size, int edges_only)
+		  double origin_y, double size, int mode)
 {
-    return gaiaSquareGridCommon (p_cache, geom, origin_x, origin_y, size,
-				 edges_only);
+    return gaiaSquareGridCommon (p_cache, geom, origin_x, origin_y, size, mode);
 }
 
 static void
@@ -973,7 +988,7 @@ get_trigrid_base (double min_x, double min_y, double origin_x, double origin_y,
 static gaiaGeomCollPtr
 gaiaTriangularGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 			  double origin_x, double origin_y, double size,
-			  int edges_only)
+			  int mode)
 {
 /* creating a regular grid [Triangular cells] */
     double min_x;
@@ -1050,7 +1065,7 @@ gaiaTriangularGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 		  {
 		      /* ok, inserting a valid cell [pointing upside] */
 		      count++;
-		      if (edges_only)
+		      if (mode > 0)
 			{
 			    /* multilinestring */
 			    ln = gaiaAddLinestringToGeomColl (result, 2);
@@ -1063,6 +1078,13 @@ gaiaTriangularGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 			    gaiaSetPoint (ln->Coords, 0, x3, y3);
 			    gaiaSetPoint (ln->Coords, 1, x1, y1);
 			}
+		      else if (mode < 0)
+			{
+			    /* multipoint */
+			    gaiaAddPointToGeomColl (result, x1, y1);
+			    gaiaAddPointToGeomColl (result, x2, y2);
+			    gaiaAddPointToGeomColl (result, x3, y3);
+			}
 		      else
 			{
 			    /* polygon */
@@ -1094,7 +1116,7 @@ gaiaTriangularGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 		  {
 		      /* ok, inserting a valid cell [pointing downside] */
 		      count++;
-		      if (edges_only)
+		      if (mode > 0)
 			{
 			    /* multilinestring */
 			    ln = gaiaAddLinestringToGeomColl (result, 2);
@@ -1107,6 +1129,13 @@ gaiaTriangularGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 			    gaiaSetPoint (ln->Coords, 0, x3, y3);
 			    gaiaSetPoint (ln->Coords, 1, x1, y1);
 			}
+		      else if (mode < 0)
+			{
+			    /* multipoint */
+			    gaiaAddPointToGeomColl (result, x1, y1);
+			    gaiaAddPointToGeomColl (result, x2, y2);
+			    gaiaAddPointToGeomColl (result, x3, y3);
+			}
 		      else
 			{
 			    /* polygon */
@@ -1139,7 +1168,7 @@ gaiaTriangularGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 	  gaiaFreeGeomColl (result);
 	  return NULL;
       }
-    if (!edges_only)
+    if (mode == 0)
       {
 	  result->DeclaredType = GAIA_MULTIPOLYGON;
 	  return result;
@@ -1152,25 +1181,27 @@ gaiaTriangularGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 	result = gaiaUnaryUnion (item);
     gaiaFreeGeomColl (item);
     result->Srid = geom->Srid;
-    result->DeclaredType = GAIA_LINESTRING;
+    if (mode < 0)
+	result->DeclaredType = GAIA_MULTIPOINT;
+    else
+	result->DeclaredType = GAIA_MULTILINESTRING;
     return result;
 }
 
 GAIAGEO_DECLARE gaiaGeomCollPtr
 gaiaTriangularGrid (gaiaGeomCollPtr geom, double origin_x, double origin_y,
-		    double size, int edges_only)
+		    double size, int mode)
 {
     return gaiaTriangularGridCommon (NULL, geom, origin_x, origin_y, size,
-				     edges_only);
+				     mode);
 }
 
 GAIAGEO_DECLARE gaiaGeomCollPtr
 gaiaTriangularGrid_r (const void *p_cache, gaiaGeomCollPtr geom,
-		      double origin_x, double origin_y, double size,
-		      int edges_only)
+		      double origin_x, double origin_y, double size, int mode)
 {
     return gaiaTriangularGridCommon (p_cache, geom, origin_x, origin_y, size,
-				     edges_only);
+				     mode);
 }
 
 static void
@@ -1245,7 +1276,7 @@ get_hexgrid_base (double min_x, double min_y, double origin_x, double origin_y,
 static gaiaGeomCollPtr
 gaiaHexagonalGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 			 double origin_x, double origin_y, double size,
-			 int edges_only)
+			 int mode)
 {
 /* creating a regular grid [Hexagonal cells] */
     double min_x;
@@ -1335,7 +1366,7 @@ gaiaHexagonalGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 		  {
 		      /* ok, inserting a valid cell */
 		      count++;
-		      if (edges_only)
+		      if (mode > 0)
 			{
 			    /* multilinestring */
 			    ln = gaiaAddLinestringToGeomColl (result, 2);
@@ -1357,6 +1388,16 @@ gaiaHexagonalGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 			    gaiaSetPoint (ln->Coords, 0, x6, y6);
 			    gaiaSetPoint (ln->Coords, 1, x1, y1);
 			}
+		      else if (mode < 0)
+			{
+			    /* multipoint */
+			    gaiaAddPointToGeomColl (result, x1, y1);
+			    gaiaAddPointToGeomColl (result, x2, y2);
+			    gaiaAddPointToGeomColl (result, x3, y3);
+			    gaiaAddPointToGeomColl (result, x4, y4);
+			    gaiaAddPointToGeomColl (result, x5, y5);
+			    gaiaAddPointToGeomColl (result, x6, y6);
+			}
 		      else
 			{
 			    /* polygon */
@@ -1394,7 +1435,7 @@ gaiaHexagonalGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 	  gaiaFreeGeomColl (result);
 	  return NULL;
       }
-    if (!edges_only)
+    if (mode == 0)
       {
 	  result->DeclaredType = GAIA_MULTIPOLYGON;
 	  return result;
@@ -1407,24 +1448,26 @@ gaiaHexagonalGridCommon (const void *p_cache, gaiaGeomCollPtr geom,
 	result = gaiaUnaryUnion (item);
     gaiaFreeGeomColl (item);
     result->Srid = geom->Srid;
-    result->DeclaredType = GAIA_LINESTRING;
+    if (mode < 0)
+	result->DeclaredType = GAIA_MULTIPOINT;
+    else
+	result->DeclaredType = GAIA_MULTILINESTRING;
     return result;
 }
 
 GAIAGEO_DECLARE gaiaGeomCollPtr
 gaiaHexagonalGrid (gaiaGeomCollPtr geom, double origin_x, double origin_y,
-		   double size, int edges_only)
+		   double size, int mode)
 {
-    return gaiaHexagonalGridCommon (NULL, geom, origin_x, origin_y, size,
-				    edges_only);
+    return gaiaHexagonalGridCommon (NULL, geom, origin_x, origin_y, size, mode);
 }
 
 GAIAGEO_DECLARE gaiaGeomCollPtr
 gaiaHexagonalGrid_r (const void *p_cache, gaiaGeomCollPtr geom, double origin_x,
-		     double origin_y, double size, int edges_only)
+		     double origin_y, double size, int mode)
 {
     return gaiaHexagonalGridCommon (p_cache, geom, origin_x, origin_y, size,
-				    edges_only);
+				    mode);
 }
 
 #endif /* end including GEOS */
