@@ -2,7 +2,7 @@
 
  gg_kml.c -- KML parser/lexer 
   
- version 4.3, 2015 June 29
+ version 5.0, 2020 August 1
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2011-2015
+Portions created by the Initial Developer are Copyright (C) 2011-2020
 the Initial Developer. All Rights Reserved.
 
 Alternatively, the contents of this file may be used under the terms of
@@ -670,6 +670,8 @@ kml_check_coord (const char *value)
 {
 /* checking a KML coordinate */
     int decimal = 0;
+    int exp = 0;
+    int expsign = 0;
     const char *p = value;
     if (*p == '+' || *p == '-')
 	p++;
@@ -684,10 +686,20 @@ kml_check_coord (const char *value)
 	    }
 	  else if (*p >= '0' && *p <= '9')
 	      ;
+	  else if (*p == 'e' || *p == 'E')
+	      exp++;
+	  else if (*p == '+' || *p == '-')
+	    {
+		if (!exp)
+		    return 0;
+		expsign++;
+	    }
 	  else
 	      return 0;
 	  p++;
       }
+    if (exp > 1 || expsign > 1)
+	return 0;
     return 1;
 }
 
@@ -1217,7 +1229,7 @@ kml_parse_polygon (struct kml_data *p_data, gaiaGeomCollPtr geom,
     int has_z;
     int inners;
     int outers;
-    int points;
+    int points = 0;
     int iv;
     int ib = 0;
     gaiaGeomCollPtr pg;
@@ -1226,7 +1238,7 @@ kml_parse_polygon (struct kml_data *p_data, gaiaGeomCollPtr geom,
     gaiaRingPtr ring;
     gaiaDynamicLinePtr dyn;
     gaiaPointPtr pt;
-    gaiaDynamicLinePtr exterior_ring;
+    gaiaDynamicLinePtr exterior_ring = NULL;
     kmlNodePtr next;
     kmlDynamicRingPtr dyn_rng;
     kmlDynamicPolygonPtr dyn_pg = kml_alloc_dyn_polygon (p_data);
@@ -1449,7 +1461,7 @@ kml_validate_geometry (struct kml_data *p_data, gaiaGeomCollPtr chain)
     int pgs = 0;
     gaiaPointPtr pt;
     gaiaLinestringPtr ln;
-    gaiaPolygonPtr pg;
+    gaiaPolygonPtr pg = NULL;
     gaiaPointPtr save_pt = NULL;
     gaiaLinestringPtr save_ln = NULL;
     gaiaPolygonPtr save_pg = NULL;
