@@ -1,7 +1,7 @@
 /* 
  gaiaexif.h -- Gaia common EXIF Metadata reading functions
   
- version 4.1, 2013 May 8
+ version 5.0, 2020 August 1
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -23,7 +23,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2013
+Portions created by the Initial Developer are Copyright (C) 2008-2020
 the Initial Developer. All Rights Reserved.
 
 Contributor(s):
@@ -83,14 +83,22 @@ extern "C"
 #define GAIA_ZIP_BLOB		6
 /** this BLOB does actually contain a PDF document */
 #define GAIA_PDF_BLOB		7
-/** this BLOB does actually contain a SpatiaLite Geometry */
+/** this BLOB does actually contain a SpatiaLite Geometry (not compressed) */
 #define GAIA_GEOMETRY_BLOB	8
+/** this BLOB does actually contain a SpatiaLite Geometry (compressed) */
+#define GAIA_COMPRESSED_GEOMETRY_BLOB	9
 /** this BLOB does actually contain a TIFF image */
-#define GAIA_TIFF_BLOB		9
+#define GAIA_TIFF_BLOB		10
 /** this BLOB does actually contain a WebP image */
-#define GAIA_WEBP_BLOB		10
+#define GAIA_WEBP_BLOB		11
+/** this BLOB does actually contain a JP2 (Jpeg2000) image */
+#define GAIA_JP2_BLOB		12
 /** this BLOB does actually contain a SpatiaLite XmlBLOB */
-#define GAIA_XML_BLOB		11
+#define GAIA_XML_BLOB		13
+/** this BLOB does actually contain a GPKG Geometry */
+#define GAIA_GPB_BLOB		14
+/** this BLOB does actually contain a SpatiaLite TinyPoint */
+#define GAIA_TINYPOINT_BLOB	15
 
 /* constants used for EXIF value types */
 /** unrecognized EXIF value */
@@ -257,7 +265,8 @@ extern "C"
 
  \sa gaiaGetExifTags, gaiaExifTagsFree
  */
-    GAIAEXIF_DECLARE gaiaExifTagPtr gaiaGetExifTagById (const gaiaExifTagListPtr
+    GAIAEXIF_DECLARE gaiaExifTagPtr gaiaGetExifTagById (const
+							gaiaExifTagListPtr
 							tag_list,
 							const unsigned short
 							tag_id);
@@ -275,8 +284,8 @@ extern "C"
     GAIAEXIF_DECLARE gaiaExifTagPtr gaiaGetExifGpsTagById (const
 							   gaiaExifTagListPtr
 							   tag_list,
-							   const unsigned short
-							   tag_id);
+							   const unsigned
+							   short tag_id);
 
 /**
  Retrieves an EXIF tag by its name
@@ -367,8 +376,10 @@ extern "C"
  \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
  gaiaExifTagGetValueType, gaiaExifTagGetNumValues
  */
-    GAIAEXIF_DECLARE unsigned char gaiaExifTagGetByteValue (const gaiaExifTagPtr
-							    tag, const int ind,
+    GAIAEXIF_DECLARE unsigned char gaiaExifTagGetByteValue (const
+							    gaiaExifTagPtr
+							    tag,
+							    const int ind,
 							    int *ok);
 
 /**
@@ -416,8 +427,9 @@ extern "C"
  \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
  gaiaExifTagGetValueType, gaiaExifTagGetNumValues
  */
-    GAIAEXIF_DECLARE unsigned int gaiaExifTagGetLongValue (const gaiaExifTagPtr
-							   tag, const int ind,
+    GAIAEXIF_DECLARE unsigned int gaiaExifTagGetLongValue (const
+							   gaiaExifTagPtr tag,
+							   const int ind,
 							   int *ok);
 
 /**
@@ -484,8 +496,9 @@ extern "C"
  \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
  gaiaExifTagGetValueType, gaiaExifTagGetNumValues
  */
-    GAIAEXIF_DECLARE short gaiaExifTagGetSignedShortValue (const gaiaExifTagPtr
-							   tag, const int ind,
+    GAIAEXIF_DECLARE short gaiaExifTagGetSignedShortValue (const
+							   gaiaExifTagPtr tag,
+							   const int ind,
 							   int *ok);
 
 /**
@@ -517,7 +530,8 @@ extern "C"
  gaiaExifTagGetValueType, gaiaExifTagGetNumValues
  */
     GAIAEXIF_DECLARE int gaiaExifTagGetSignedRational1Value (const
-							     gaiaExifTagPtr tag,
+							     gaiaExifTagPtr
+							     tag,
 							     const int ind,
 							     int *ok);
 
@@ -534,7 +548,8 @@ extern "C"
  gaiaExifTagGetValueType, gaiaExifTagGetNumValues
  */
     GAIAEXIF_DECLARE int gaiaExifTagGetSignedRational2Value (const
-							     gaiaExifTagPtr tag,
+							     gaiaExifTagPtr
+							     tag,
 							     const int ind,
 							     int *ok);
 
@@ -583,8 +598,9 @@ extern "C"
  \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
  gaiaExifTagGetValueType, gaiaExifTagGetNumValues
  */
-    GAIAEXIF_DECLARE double gaiaExifTagGetDoubleValue (const gaiaExifTagPtr tag,
-						       const int ind, int *ok);
+    GAIAEXIF_DECLARE double gaiaExifTagGetDoubleValue (const gaiaExifTagPtr
+						       tag, const int ind,
+						       int *ok);
 
 /**
  Return a human readable description from an EXIF tag
@@ -596,9 +612,9 @@ extern "C"
 
  \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName
  */
-    GAIAEXIF_DECLARE void gaiaExifTagGetHumanReadable (const gaiaExifTagPtr tag,
-						       char *str, int len,
-						       int *ok);
+    GAIAEXIF_DECLARE void gaiaExifTagGetHumanReadable (const gaiaExifTagPtr
+						       tag, char *str,
+						       int len, int *ok);
 
 /**
  Attempts to guess the actual content-type of some BLOB
@@ -608,7 +624,8 @@ extern "C"
 
  \return the BLOB type: one of GAIA_HEX_BLOB, GAIA_GIF_BLOB, GAIA_PNG_BLOB,
  GAIA_JPEG_BLOB, GAIA_EXIF_BLOB, GAIA_EXIF_GPS_BLOB, GAIA_ZIP_BLOB,
- GAIA_PDF_BLOB, GAIA_GEOMETRY_BLOB, GAIA_TIFF_BLOB
+ GAIA_PDF_BLOB, GAIA_GEOMETRY_BLOB, GAIA_TIFF_BLOB, GAIA_WEBP_BLOB,
+ GAIA_JP2_BLOB, GAIA_XML_BLOB, GAIA_GPB_BLOB
  */
     GAIAEXIF_DECLARE int gaiaGuessBlobType (const unsigned char *blob,
 					    int size);
@@ -625,8 +642,9 @@ extern "C"
  \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
  gaiaIsExifGpsTag
  */
-    GAIAEXIF_DECLARE int gaiaGetGpsCoords (const unsigned char *blob, int size,
-					   double *longitude, double *latitude);
+    GAIAEXIF_DECLARE int gaiaGetGpsCoords (const unsigned char *blob,
+					   int size, double *longitude,
+					   double *latitude);
 /**
  Return a text string representing DMS coordinates from an EXIF-GPS tag
 
@@ -640,8 +658,9 @@ extern "C"
  \sa gaiaGetExifTagById, gaiaGetExifGpsTagById, gaiaGetExifTagByName, 
  gaiaIsExifGpsTag
  */
-    GAIAEXIF_DECLARE int gaiaGetGpsLatLong (const unsigned char *blob, int size,
-					    char *latlong, int ll_size);
+    GAIAEXIF_DECLARE int gaiaGetGpsLatLong (const unsigned char *blob,
+					    int size, char *latlong,
+					    int ll_size);
 
 #ifdef __cplusplus
 }
