@@ -2,7 +2,7 @@
 
  virtualbbox.c -- SQLite3 extension [VIRTUAL TABLE accessing BoundingBox tables]
 
- version 4.3, 2015 June 29
+ version 5.0, 2020 August 1
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2015
+Portions created by the Initial Developer are Copyright (C) 2008-2020
 the Initial Developer. All Rights Reserved.
 
 Contributor(s):
@@ -307,7 +307,7 @@ vbbox_read_row (VirtualBBoxCursorPtr cursor)
     double miny = DBL_MAX;
     double maxx = 0.0 - DBL_MAX;
     double maxy = 0.0 - DBL_MAX;
-    int srid;
+    int srid = -1;
     char ok_minx = 'N';
     char ok_miny = 'N';
     char ok_maxx = 'N';
@@ -711,7 +711,7 @@ static int
 vbbox_open (sqlite3_vtab * pVTab, sqlite3_vtab_cursor ** ppCursor)
 {
 /* opening a new cursor */
-    sqlite3_stmt *stmt;
+    sqlite3_stmt *stmt = NULL;
     gaiaOutBuffer sql_statement;
     int ret;
     char *sql;
