@@ -2,7 +2,7 @@
 
  virtualelementary.c -- SQLite3 extension [VIRTUAL TABLE Elementary Geometries]
 
- version 4.3, 2015 June 29
+ version 5.0, 2020 August 1
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2015
+Portions created by the Initial Developer are Copyright (C) 2008-2020
 the Initial Developer. All Rights Reserved.
 
 Contributor(s):
@@ -633,7 +633,7 @@ velem_filter (sqlite3_vtab_cursor * pCursor, int idxNum, const char *idxStr,
     char *table_nameQ;
     char *geo_nameQ;
     char *sql_statement;
-    sqlite3_int64 rowid_value;
+    sqlite3_int64 rowid_value = 0;
     int ok_prefix = 0;
     int ok_table = 0;
     int ok_geom = 0;
