@@ -2,7 +2,7 @@
 
  virtualgpkg.c -- SQLite3 extension [VIRTUAL TABLE accessing GPKG tables]
 
- version 4.3, 2015 June 29
+ version 5.0, 2020 August 1
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2015
+Portions created by the Initial Developer are Copyright (C) 2015-2020
 the Initial Developer. All Rights Reserved.
 
 Contributor(s):
@@ -75,6 +75,10 @@ the terms of any one of the MPL, the GPL or the LGPL.
 #define LONG64_MIN	(-LONG64_MAX + 1)
 #endif
 
+#ifdef _WIN32
+#define strcasecmp	_stricmp
+#endif /* not WIN32 */
+
 #ifdef ENABLE_GEOPACKAGE	/* only if GeoPackage support is enabled */
 
 struct sqlite3_module my_gpkg_module;
@@ -98,6 +102,7 @@ typedef struct VirtualGPKGStruct
     int nRef;			/* # references: USED INTERNALLY BY SQLITE */
     char *zErrMsg;		/* error message: USE INTERNALLY BY SQLITE */
     sqlite3 *db;		/* the sqlite db holding the virtual table */
+    char *db_prefix;		/* the prefix identifying the ATTACHED-DB where the table is */
     char *table;		/* the real-table name */
     int nColumns;		/* the # columns into the table */
     char **Column;		/* the name for each column */
@@ -298,11 +303,14 @@ vgpkg_insert_row (VirtualGPKGPtr p_vt, sqlite3_int64 * rowid, int argc,
     int size;
     char *sql;
     char buf[256];
+    char *xprefix;
     char *xname;
     gaiaOutBufferInitialize (&sql_statement);
+    xprefix = gaiaDoubleQuotedSql (p_vt->db_prefix);
     xname = gaiaDoubleQuotedSql (p_vt->table);
-    sql = sqlite3_mprintf ("INSERT INTO \"%s\" ", xname);
+    sql = sqlite3_mprintf ("INSERT INTO \"%s\".\"%s\" ", xprefix, xname);
     free (xname);
+    free (xprefix);
     gaiaAppendToOutBuffer (&sql_statement, sql);
     sqlite3_free (sql);
     for (ic = 0; ic < p_vt->nColumns; ic++)
@@ -395,11 +403,14 @@ vgpkg_update_row (VirtualGPKGPtr p_vt, sqlite3_int64 rowid, int argc,
     int size;
     char *sql;
     char buf[256];
+    char *xprefix;
     char *xname;
     gaiaOutBufferInitialize (&sql_statement);
+    xprefix = gaiaDoubleQuotedSql (p_vt->db_prefix);
     xname = gaiaDoubleQuotedSql (p_vt->table);
-    sql = sqlite3_mprintf ("UPDATE \"%s\" SET", xname);
+    sql = sqlite3_mprintf ("UPDATE \"%s\".\"%s\" SET", xprefix, xname);
     free (xname);
+    free (xprefix);
     gaiaAppendToOutBuffer (&sql_statement, sql);
     sqlite3_free (sql);
     for (ic = 0; ic < p_vt->nColumns; ic++)
@@ -477,12 +488,15 @@ vgpkg_delete_row (VirtualGPKGPtr p_vt, sqlite3_int64 rowid)
     char *sql_statement;
     char dummy[256];
     int ret;
+    char *xprefix;
     char *xname;
+    xprefix = gaiaDoubleQuotedSql (p_vt->db_prefix);
     xname = gaiaDoubleQuotedSql (p_vt->table);
     sprintf (dummy, FRMT64, rowid);
     sql_statement =
 	sqlite3_mprintf ("DELETE FROM \"%s\" WHERE ROWID = %s", xname, dummy);
     free (xname);
+    free (xprefix);
     ret = sqlite3_exec (p_vt->db, sql_statement, NULL, NULL, NULL);
     sqlite3_free (sql_statement);
     return ret;
@@ -495,6 +509,8 @@ free_table (VirtualGPKGPtr p_vt)
     int i;
     if (!p_vt)
 	return;
+    if (p_vt->db_prefix)
+	sqlite3_free (p_vt->db_prefix);
     if (p_vt->table)
 	sqlite3_free (p_vt->table);
     if (p_vt->Column)
@@ -537,6 +553,7 @@ vgpkg_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
 {
 /* creates the virtual table connected to some GPKG table */
     char *vtable = NULL;
+    char *db_prefix = NULL;
     char *table = NULL;
     int ret;
     int i;
@@ -552,6 +569,7 @@ vgpkg_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
     char **results;
     char *sql;
     char prefix[16];
+    char *xdb_prefix;
     char *xname;
     gaiaOutBuffer sql_statement;
     VirtualGPKGPtr p_vt = NULL;
@@ -562,8 +580,15 @@ vgpkg_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
     if (argc == 4)
       {
 	  vtable = gaiaDequotedSql ((char *) argv[2]);
+	  db_prefix = gaiaDequotedSql ("main");
 	  table = gaiaDequotedSql ((char *) argv[3]);
       }
+    else if (argc == 5)
+      {
+	  vtable = gaiaDequotedSql ((char *) argv[2]);
+	  db_prefix = gaiaDequotedSql ((char *) argv[3]);
+	  table = gaiaDequotedSql ((char *) argv[4]);
+      }
     else
       {
 	  *pzErr =
@@ -572,9 +597,12 @@ vgpkg_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
 	  goto error;
       }
 /* retrieving the base table columns */
+    xdb_prefix = gaiaDoubleQuotedSql (db_prefix);
     xname = gaiaDoubleQuotedSql (table);
-    sql = sqlite3_mprintf ("PRAGMA table_info(\"%s\")", xname);
+    sql =
+	sqlite3_mprintf ("PRAGMA \"%s\".table_info(\"%s\")", xdb_prefix, xname);
     free (xname);
+    free (xdb_prefix);
     ret = sqlite3_get_table (db, sql, &results, &n_rows, &n_columns, NULL);
     sqlite3_free (sql);
     if (ret != SQLITE_OK)
@@ -587,6 +615,9 @@ vgpkg_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
 	  p_vt->db = db;
 	  p_vt->nRef = 0;
 	  p_vt->zErrMsg = NULL;
+	  len = strlen (db_prefix);
+	  p_vt->db_prefix = sqlite3_malloc (len + 1);
+	  strcpy (p_vt->db_prefix, db_prefix);
 	  len = strlen (table);
 	  p_vt->table = sqlite3_malloc (len + 1);
 	  strcpy (p_vt->table, table);
@@ -626,11 +657,13 @@ vgpkg_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
     else
 	goto illegal;
 /* retrieving the base table columns */
+    xdb_prefix = gaiaDoubleQuotedSql (db_prefix);
     sql =
 	sqlite3_mprintf
 	("SELECT column_name, geometry_type_name, srs_id, z, m\n"
-	 "FROM gpkg_geometry_columns WHERE Upper(table_name) = Upper(%Q)",
-	 table);
+	 "FROM \"%s\".gpkg_geometry_columns WHERE Upper(table_name) = Upper(%Q)",
+	 xdb_prefix, table);
+    free (xdb_prefix);
     ret = sqlite3_get_table (db, sql, &results, &n_rows, &n_columns, NULL);
     sqlite3_free (sql);
     if (ret != SQLITE_OK)
@@ -727,9 +760,11 @@ vgpkg_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
     else
 	goto illegal;
 /* preparing the COLUMNs for this VIRTUAL TABLE */
+    xdb_prefix = gaiaDoubleQuotedSql (db_prefix);
     xname = gaiaDoubleQuotedSql (vtable);
-    sql = sqlite3_mprintf ("CREATE TABLE \"%s\" ", xname);
+    sql = sqlite3_mprintf ("CREATE TABLE \"%s\".\"%s\" ", xdb_prefix, xname);
     free (xname);
+    free (xdb_prefix);
     gaiaAppendToOutBuffer (&sql_statement, sql);
     sqlite3_free (sql);
     for (i = 0; i < p_vt->nColumns; i++)
@@ -767,6 +802,7 @@ vgpkg_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
 	goto error;
     *ppVTab = (sqlite3_vtab *) p_vt;
     free (vtable);
+    free (db_prefix);
     free (table);
     return SQLITE_OK;
   illegal:
@@ -781,6 +817,8 @@ vgpkg_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
   error:
     if (vtable)
 	free (vtable);
+    if (db_prefix)
+	free (db_prefix);
     if (table)
 	free (table);
     gaiaOutBufferReset (&sql_statement);
@@ -829,6 +867,7 @@ vgpkg_open (sqlite3_vtab * pVTab, sqlite3_vtab_cursor ** ppCursor)
     int ret;
     char *sql;
     int ic;
+    char *xprefix;
     char *xname;
     VirtualGPKGCursorPtr cursor =
 	(VirtualGPKGCursorPtr) sqlite3_malloc (sizeof (VirtualGPKGCursor));
@@ -853,9 +892,11 @@ vgpkg_open (sqlite3_vtab * pVTab, sqlite3_vtab_cursor ** ppCursor)
 	  gaiaAppendToOutBuffer (&sql_statement, sql);
 	  sqlite3_free (sql);
       }
+    xprefix = gaiaDoubleQuotedSql (cursor->pVtab->db_prefix);
     xname = gaiaDoubleQuotedSql (cursor->pVtab->table);
     sql = sqlite3_mprintf (" FROM \"%s\" WHERE ROWID >= ?", xname);
     free (xname);
+    free (xprefix);
     gaiaAppendToOutBuffer (&sql_statement, sql);
     sqlite3_free (sql);
     if (sql_statement.Error == 0 && sql_statement.Buffer != NULL)
