@@ -2,7 +2,7 @@
 
  virtualxpath.c -- SQLite3 extension [VIRTUAL XPath handler]
 
- version 4.3, 2015 June 29
+ version 5.0, 2020 August 1
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2015
+Portions created by the Initial Developer are Copyright (C) 2008-2020
 the Initial Developer. All Rights Reserved.
 
 Contributor(s):
@@ -615,7 +615,7 @@ vxpath_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
     return SQLITE_OK;
   illegal:
 /* something is going the wrong way */
-    if (!okTable == 0)
+    if (!okTable)
 	*pzErr =
 	    sqlite3_mprintf
 	    ("[VirtualXPath module] table \"%s\" doesn't exists\n", table);
