@@ -47,6 +47,7 @@ the terms of any one of the MPL, the GPL or the LGPL.
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <math.h>
 
 #include "config.h"
 
@@ -819,7 +820,7 @@ test_current_mode ()
       }
 
     resultDouble = gaiaLineLocatePoint_r (cache, 0, validGeometry);
-    if (abs (resultDouble - -1.0) > double_eps)
+    if (fabs (resultDouble - -1.0) > double_eps)
       {
 	  fprintf (stderr, "bad result at %s:%i: %f\n", __FILE__, __LINE__,
 		   resultDouble);
@@ -828,7 +829,7 @@ test_current_mode ()
       }
 
     resultDouble = gaiaLineLocatePoint_r (cache, validGeometry, 0);
-    if (abs (resultDouble - -1.0) > double_eps)
+    if (fabs (resultDouble - -1.0) > double_eps)
       {
 	  fprintf (stderr, "bad result at %s:%i: %f\n", __FILE__, __LINE__,
 		   resultDouble);
@@ -837,7 +838,7 @@ test_current_mode ()
       }
 
     resultDouble = gaiaLineLocatePoint_r (cache, 0, 0);
-    if (abs (resultDouble - -1.0) > double_eps)
+    if (fabs (resultDouble - -1.0) > double_eps)
       {
 	  fprintf (stderr, "bad result at %s:%i: %f\n", __FILE__, __LINE__,
 		   resultDouble);
@@ -1711,7 +1712,7 @@ test_legacy_mode ()
       }
 
     resultDouble = gaiaLineLocatePoint (0, validGeometry);
-    if (abs (resultDouble - -1.0) > double_eps)
+    if (fabs (resultDouble - -1.0) > double_eps)
       {
 	  fprintf (stderr, "bad result at %s:%i: %f\n", __FILE__, __LINE__,
 		   resultDouble);
@@ -1720,7 +1721,7 @@ test_legacy_mode ()
       }
 
     resultDouble = gaiaLineLocatePoint (validGeometry, 0);
-    if (abs (resultDouble - -1.0) > double_eps)
+    if (fabs (resultDouble - -1.0) > double_eps)
       {
 	  fprintf (stderr, "bad result at %s:%i: %f\n", __FILE__, __LINE__,
 		   resultDouble);
@@ -1729,7 +1730,7 @@ test_legacy_mode ()
       }
 
     resultDouble = gaiaLineLocatePoint (0, 0);
-    if (abs (resultDouble - -1.0) > double_eps)
+    if (fabs (resultDouble - -1.0) > double_eps)
       {
 	  fprintf (stderr, "bad result at %s:%i: %f\n", __FILE__, __LINE__,
 		   resultDouble);
@@ -2100,6 +2101,8 @@ test_legacy_mode ()
     gaiaSetPoint (rng->Coords, 3, 4.5, 30.5);
     gaiaSetPoint (rng->Coords, 4, 4.5, 4.5);
 
+
+#ifndef GEOS_ONLY_REENTRANT	/* skipping legacy mode test (non-thread-safe GEOS API) */
     /* Tests Polygons UnaryUnion [as in aggregate ST_Union] */
     g = gaiaUnaryUnion (geom);
     if (g == NULL)
@@ -2141,6 +2144,7 @@ test_legacy_mode ()
 	  goto exit;
       }
     gaiaFreeGeomColl (g);
+#endif
     gaiaFreeGeomColl (geom);
 
     /* Cleanup and exit */
